/* Copyright (C) 2008 by H-Store Project
 * Brown University
 * Massachusetts Institute of Technology
 * Yale University
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

// I'm in your code messing it up Evan!

#ifndef DEBUGLOG_H__
#define DEBUGLOG_H__

/**
 * Debug logging functions for EE. Unlike the performance counters,
 * these are just printf() turned on/off by LOG_LEVEL compile option.
 * The main concern here is not to add any overhead on runtime performance
 * when the logging is turned off. Use LOG_XXX_ENABLED macros defined here to
 * eliminate all instructions in the final binary.
*/

#include <string>
#include <ctime>
#include <cstdio>

// Log levels.
#define LOG_LEVEL_OFF    1000
#define LOG_LEVEL_ERROR  500
#define LOG_LEVEL_WARN   400
#define LOG_LEVEL_INFO   300
#define LOG_LEVEL_DEBUG  200
#define LOG_LEVEL_TRACE  100
#define LOG_LEVEL_ALL    0

// #define LOG_TIME_FORMAT "%Y-%m-%d %H:%M:%S"
#define LOG_TIME_FORMAT "%X"
#define LOG_OUTPUT_STREAM stderr

// Compile Option
#ifndef LOG_LEVEL
    // TODO : any way to use pragma message in GCC?
    //#pragma message("Warning: LOG_LEVEL compile option was not explicitly given.")
    #if defined(DEBUG) || defined(_DEBUG) || defined(_DEBUG_)
        //#pragma message("LOG_LEVEL_DEBUG is used instead as DEBUG option is on.")
        #define LOG_LEVEL LOG_LEVEL_DEBUG
    #else
        //#pragma message("LOG_LEVEL_WARN is used instead as DEBUG option is off.")
        #define LOG_LEVEL LOG_LEVEL_INFO
    #endif
    //#pragma message("Give LOG_LEVEL compile option to overwrite the default level.")
#endif


// For compilers which do not support __FUNCTION__
#if !defined(__FUNCTION__) && !defined(__GNUC__)
    #define __FUNCTION__ ""
#endif

void outputLogHeader_(const char *file, int line, const char *func, int level);

// Two convenient macros for debugging
// 1. Logging macros.
// 2. LOG_XXX_ENABLED macros. Use these to "eliminate" all the debug blocks from release binary.
#ifdef LOG_ERROR_ENABLED
    #undef LOG_ERROR_ENABLED
#endif
#if LOG_LEVEL<=LOG_LEVEL_ERROR
    #define LOG_ERROR_ENABLED
    //#pragma message("LOG_ERROR was enabled.")
    #define LOG_ERROR(...) outputLogHeader_(__FILE__, __LINE__, __FUNCTION__, LOG_LEVEL_ERROR);::fprintf(LOG_OUTPUT_STREAM, __VA_ARGS__);fprintf(LOG_OUTPUT_STREAM, "\n");::fflush(LOG_OUTPUT_STREAM)
#else
    #define LOG_ERROR(...) ((void)0)
#endif

#ifdef LOG_WARN_ENABLED
    #undef LOG_WARN_ENABLED
#endif
#if LOG_LEVEL<=LOG_LEVEL_WARN
    #define LOG_WARN_ENABLED
    //#pragma message("LOG_WARN was enabled.")
    #define LOG_WARN(...) outputLogHeader_(__FILE__, __LINE__, __FUNCTION__, LOG_LEVEL_WARN);::fprintf(LOG_OUTPUT_STREAM, __VA_ARGS__);fprintf(LOG_OUTPUT_STREAM, "\n");::fflush(LOG_OUTPUT_STREAM)
#else
    #define LOG_WARN(...) ((void)0)
#endif

#ifdef LOG_INFO_ENABLED
    #undef LOG_INFO_ENABLED
#endif
#if LOG_LEVEL<=LOG_LEVEL_INFO
    #define LOG_INFO_ENABLED
    //#pragma message("LOG_INFO was enabled.")
    #define LOG_INFO(...) outputLogHeader_(__FILE__, __LINE__, __FUNCTION__, LOG_LEVEL_INFO);::fprintf(LOG_OUTPUT_STREAM, __VA_ARGS__);fprintf(LOG_OUTPUT_STREAM, "\n");::fflush(LOG_OUTPUT_STREAM)
#else
    #define LOG_INFO(...) ((void)0)
#endif

#ifdef LOG_DEBUG_ENABLED
    #undef LOG_DEBUG_ENABLED
#endif
#if LOG_LEVEL<=LOG_LEVEL_DEBUG
    #define LOG_DEBUG_ENABLED
    //#pragma message("LOG_DEBUG was enabled.")
    #define LOG_DEBUG(...) outputLogHeader_(__FILE__, __LINE__, __FUNCTION__, LOG_LEVEL_DEBUG);::fprintf(LOG_OUTPUT_STREAM, __VA_ARGS__);fprintf(LOG_OUTPUT_STREAM, "\n");::fflush(LOG_OUTPUT_STREAM)
#else
    #define LOG_DEBUG(...) ((void)0)
#endif

#ifdef LOG_TRACE_ENABLED
    #undef LOG_TRACE_ENABLED
#endif
#if LOG_LEVEL<=LOG_LEVEL_TRACE
    #define LOG_TRACE_ENABLED
    //#pragma message("LOG_TRACE was enabled.")
    #define LOG_TRACE(...) outputLogHeader_(__FILE__, __LINE__, __FUNCTION__, LOG_LEVEL_TRACE);::fprintf(LOG_OUTPUT_STREAM, __VA_ARGS__);fprintf(LOG_OUTPUT_STREAM, "\n");::fflush(LOG_OUTPUT_STREAM)
#else
    #define LOG_TRACE(...) ((void)0)
#endif

// Output log message header in this format: [type] [file:line:function] time -
// ex: [ERROR] [somefile.cpp:123:doSome()] 2008/07/06 10:00:00 -
inline void outputLogHeader_(const char *file, int line, const char *func, int level) {
    time_t t = ::time(NULL) ;
    tm *curTime = localtime(&t);
    char time_str[32]; // FIXME
    ::strftime(time_str, 32, LOG_TIME_FORMAT, curTime);
    const char* type;
    switch (level) {
        case LOG_LEVEL_ERROR:
            type = "ERROR";
            break;
        case LOG_LEVEL_WARN:
            type = "WARN ";
            break;
        case LOG_LEVEL_INFO:
            type = "INFO ";
            break;
        case LOG_LEVEL_DEBUG:
            type = "DEBUG";
            break;
        case LOG_LEVEL_TRACE:
            type = "TRACE";
            break;
        default:
            type = "UNKWN";
    }
    fprintf(LOG_OUTPUT_STREAM, "%s [%s:%d] %s - ", time_str, file, line, type);
}

#endif
