/* This file is part of VoltDB.
 * Copyright (C) 2008-2010 VoltDB Inc.
 *
 * This file contains original code and/or modifications of original code.
 * Any modifications made by VoltDB Inc. are licensed under the following
 * terms and conditions:
 *
 * VoltDB is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VoltDB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VoltDB.  If not, see <http://www.gnu.org/licenses/>.
 */
/* Copyright (C) 2008 by H-Store Project
 * Brown University
 * Massachusetts Institute of Technology
 * Yale University
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef HSTOREJOINNODE_H
#define HSTOREJOINNODE_H

#include "abstractplannode.h"

namespace voltdb
{

class AbstractExpression;

class AbstractJoinPlanNode : public AbstractPlanNode
{
public:
    AbstractJoinPlanNode(CatalogId id);
    AbstractJoinPlanNode();
    virtual ~AbstractJoinPlanNode();

    void setJoinType(JoinType join_type);
    JoinType getJoinType() const;

    void setPredicate(AbstractExpression* predicate);
    AbstractExpression* getPredicate() const;

    virtual std::string debugInfo(const std::string& spacer) const;

    virtual int getColumnIndexFromGuid(int guid,
                                       const catalog::Database* db) const;

    void setOutputColumnGuids(std::vector<int> output_column_guids);

protected:
    friend AbstractPlanNode*
        AbstractPlanNode::fromJSONObject(json_spirit::Object& obj,
                                         const catalog::Database *catalog_db);

    virtual void loadFromJSONObject(json_spirit::Object& obj,
                                    const catalog::Database *catalog_db);

    //
    // This is the predicate to figure out whether a joined tuple should
    // be put into the output table
    //
    AbstractExpression* m_predicate;
    //
    // We currently don't do anything with this...
    //
    JoinType m_joinType;

    /*
     * Initialized by the executors p_init. It has to generate the
     * schema for the output table anyways so we use that opportunity
     * to cache the output column names and indexes in this vector so
     * it can be retrieved by other plan nodes/executors
     */
    std::vector<int> m_outputColumnGuids;
};

}

#endif
